//--------------------------------------------------------------
// File     : stm32_ub_rn42.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_RN42_H
#define __STM32F4_UB_RN42_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_usart.h" 
#include "misc.h"



//--------------------------------------------------------------
// UART Defines
// COM3, 115200 Baud, TX=PC10, RX=PC11
//--------------------------------------------------------------

#define  RN42_UART_CLOCK   RCC_APB1Periph_USART3  // UART
#define  RN42_UART_AF      GPIO_AF_USART3
#define  RN42_UART_COM     USART3
#define  RN42_UART_BAUD    115200
#define  RN42_UART_INT     USART3_IRQn
#define  RN42_UART_IRQ     USART3_IRQHandler

#define  RN42_TX_CLOCK     RCC_AHB1Periph_GPIOC   // TX-Pin
#define  RN42_TX_PORT      GPIOC
#define  RN42_TX_PIN       GPIO_Pin_10
#define  RN42_TX_SOURCE    GPIO_PinSource10

#define  RN42_RX_CLOCK     RCC_AHB1Periph_GPIOC   // RX-Pin
#define  RN42_RX_PORT      GPIOC
#define  RN42_RX_PIN       GPIO_Pin_11
#define  RN42_RX_SOURCE    GPIO_PinSource11




//--------------------------------------------------------------
// Status vom Mode
//--------------------------------------------------------------
typedef enum {
  RN42_MODE_ERROR = 0,  // Error
  RN42_MODE_CMD,        // Command-Mode
  RN42_MODE_DATA        // Data-Mode
}RN42_MODE_t;





//--------------------------------------------------------------
// Status beim Empfangen
//--------------------------------------------------------------
typedef enum {
  RN42_RX_EMPTY = 0,  // nichts empfangen
  RN42_RX_READY,      // es steht was im Empfangspuffer
  RN42_RX_FULL        // RX-Puffer ist voll
}RN42_RXSTATUS_t;



//--------------------------------------------------------------
// Endekennung beim Senden
//--------------------------------------------------------------
typedef enum {
  RN42_NONE = 0,  // keine Endekennung
  RN42_LFCR,      // LineFeed + CarriageReturn (0x0A,0x0D)
  RN42_CRLF,      // CarriageReturn + LineFeed (0x0D,0x0A)
  RN42_LF,        // nur LineFeed (0x0A)
  RN42_CR         // nur CarriageReturn (0x0D)
}RN42_LASTBYTE_t;



//--------------------------------------------------------------
// Defines fuer das Empfangen
//--------------------------------------------------------------
#define  RN42_RX_BUF_SIZE   50    // Grsse vom RX-Puffer in Bytes
#define  RN42_RX_FIRST_CHR  0x20  // erstes erlaubte Zeichen (Ascii-Wert)
#define  RN42_RX_LAST_CHR   0x7E  // letztes erlaubt Zeichen (Ascii-Wert)
#define  RN42_RX_END_CHR    0x0D  // Endekennung (Ascii-Wert)

#define  RN42_INIT_DELAY   200000   // kleine Pause beim init
#define  RN42_RX_TIMEOUT   2000000  // Timeout (>1000000)



//--------------------------------------------------------------
// Struktur vom RN42-Modul
//--------------------------------------------------------------
typedef struct {
  RN42_MODE_t mode;
  char buffer[RN42_RX_BUF_SIZE];
}RN42_t;
RN42_t RN42;


//--------------------------------------------------------------
// Struktur fr RN42_RX
//--------------------------------------------------------------
typedef struct {
  char rx_buffer[RN42_RX_BUF_SIZE]; // RX-Puffer
  uint8_t wr_ptr;                   // Schreib Pointer
  RN42_RXSTATUS_t status;           // RX-Status
}RN42_RX_t;
RN42_RX_t RN42_RX;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
ErrorStatus UB_RN42_Init(void);
ErrorStatus UB_RN42_DATA_Mode(void);
ErrorStatus UB_RN42_CMD_Mode(void);
RN42_RXSTATUS_t UB_RN42_ReceiveString(char *ptr);
ErrorStatus UB_RN42_SendString(char *ptr, RN42_LASTBYTE_t last_byte);
ErrorStatus UB_RN42_Get_BT_Adr(void);
ErrorStatus UB_RN42_Set_BT_Adr(char *ptr);
ErrorStatus UB_RN42_Set_BT_Pin(char *ptr);
ErrorStatus UB_RN42_Set_BT_Name(char *ptr);
ErrorStatus UB_RN42_Connect(void);
ErrorStatus UB_RN42_Disconnect(void);



//--------------------------------------------------------------
#endif // __STM32F4_UB_RN42_H
